/*
 Scalarprod.c
 ============
 
 This program demonstrates a simple scalarproduct implementation in C,
 using two large and randomly initialized vectors.
 
 Author: S. Winberg, 2010
*/

// Standard library includes
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <pthread.h>

// Utilities for using timers
#include "timeutils.h"

#define VECTOR_LEN 1024

float *glob_a, *glob_b;
float glob_sum[2];

float random_f ()
//returns a random number within the given boundary
{ 
	return (float)rand() / RAND_MAX;
}

void* scalar_prod (void* arg)
{
      int start = (int)arg;
      float sum = 0;
      int i;
      printf("start %d\n",arg);
      for (i=start; i<VECTOR_LEN; i+=2) sum = sum + (glob_a[i] * glob_b[i]);
      glob_sum[start]=sum;
}

int main ( int argc, char* argv[] )
{
    // declare variables
    unsigned long long t0,t1;
    int i;
    float sum;
    pthread_t th1, th2;
    
    // calculate the CPU speed
    int cpu_speed = avg_cpu_speed(2);

    // create two vectors of VECTOR_LEN floats
    glob_a = malloc(sizeof(float)*VECTOR_LEN);
    glob_b = malloc(sizeof(float)*VECTOR_LEN);
    
    // create a random seed
    srand((int) time(0));
    
    // start processing
    t0 = CPU_ticks();
    i  = 0;
    // Do processing
    
    // first initialize the vectors
    for (i=0; i<VECTOR_LEN; i++) {
          glob_a[i] = random_f(); // get a random value and store in vector a
          glob_b[i] = random_f(); // get a random value and store in vector b
       }
       
    pthread_create( &th1, NULL, scalar_prod, (void*) 0);
    pthread_create( &th2, NULL, scalar_prod, (void*) 1);
    
    // Synchronize completion of each thread.
	pthread_join(th1,NULL);
	pthread_join(th2,NULL);
    
    // get the time elapsed
    t1 = CPU_ticks();
    
    free(glob_a); free(glob_b);
    
    // combine results
    sum = glob_sum[0] + glob_sum[1];
    
    // print results
    printf("vectorprod = %f\n",sum);
    
    // print memory budget
    unsigned msize = sizeof(float)*VECTOR_LEN;
    printf("Memory usage\n"
      " Matrices each: %u bytes\n Total: %u bytes (%f Megabytes)\n",
      msize,2*msize,(float)(2.0*msize)/(1024.0*1024.0)
      );
    
    // show perf stats
    show_rdtsc_stats(t0,t1);
    
    
    system("pause");
    
    return 0;
}

