/*
  Timeutils.h
  This module provides (hopefully quite portable) timing routines.
  
  Call time_now() to return a milliseconds count
  Call measure_clock_speed(int n_avg) to calculate the CPU's clock speed, where
                              n = number of 1s averages to do. Uses Intel high
                              performance counter register. Don't call if not
                              using an Intel CPU.
                              Suggested: n_avg = 2
*/

#ifndef TIMERUTILS_H
#define TIMERUTILS_H 1

/* Function time_now returns a value in milliseconds indicating current time */
long time_now (void);


/* Function waits 1s and measures the CPU time using INTEL high performance. */
unsigned long long measure_clock_speed ();

/* Return the CPU speed as a number of MHz */
int avg_cpu_speed (int n_avg);

/* Read the Intel high performance counter register */
unsigned long long int CPU_ticks ();

void show_rdtsc_stats( 
     unsigned long long int time0,
     unsigned long long int time1 );

#endif
